@@ -1,4 +1,4 @@
-/*      $Id: irrecord.c,v 5.55 2005/08/26 20:01:00 lirc Exp $      */
+/*      $Id: irrecord.c,v 5.75 2008/09/21 10:36:51 lirc Exp $      */
 
 /****************************************************************************
  ** irrecord.c **************************************************************
@@ -51,37 +51,39 @@
 #include "dump_config.h"
 #include "ir_remote.h"
 #include "config_file.h"
+#include "receive.h"
 
 void flushhw(void);
 int resethw(void);
 int waitfordata(unsigned long maxusec);
 int availabledata(void);
-int get_toggle_bit(struct ir_remote *remote);
-void set_toggle_bit(struct ir_remote *remote,ir_code xor);
+int get_toggle_bit_mask(struct ir_remote *remote);
+void set_toggle_bit_mask(struct ir_remote *remote,ir_code xor);
 void get_pre_data(struct ir_remote *remote);
 void get_post_data(struct ir_remote *remote);
-#ifdef DEBUG
 typedef void (*remote_func)(struct ir_remote *remotes);
 
 void for_each_remote(struct ir_remote *remotes, remote_func func);
+void analyse_remote(struct ir_remote *raw_data);
+#ifdef DEBUG
 void remove_pre_data(struct ir_remote *remote);
 void remove_post_data(struct ir_remote *remote);
 void invert_data(struct ir_remote *remote);
 void remove_trail(struct ir_remote *remote);
 #endif
-int get_lengths(struct ir_remote *remote,int force);
+int get_lengths(struct ir_remote *remote, int force, int interactive);
 struct lengths *new_length(lirc_t length);
 int add_length(struct lengths **first,lirc_t length);
-void free_lengths(struct lengths *first);
+void free_lengths(struct lengths **firstp);
 void merge_lengths(struct lengths *first);
-void get_scheme(struct ir_remote *remote);
+void get_scheme(struct ir_remote *remote, int interactive);
 struct lengths *get_max_length(struct lengths *first,unsigned int *sump);
 void unlink_length(struct lengths **first,struct lengths *remove);
-int get_trail_length(struct ir_remote *remote);
+int get_trail_length(struct ir_remote *remote, int interactive);
 int get_lead_length(struct ir_remote *remote);
-int get_repeat_length(struct ir_remote *remote);
-int get_header_length(struct ir_remote *remote);
-int get_data_length(struct ir_remote *remote);
+int get_repeat_length(struct ir_remote *remote, int interactive);
+int get_header_length(struct ir_remote *remote, int interactive);
+int get_data_length(struct ir_remote *remote, int interactive);
 int get_gap_length(struct ir_remote *remote);
 void fprint_copyright(FILE *fout);
 
@@ -108,8 +110,8 @@ struct ir_ncode ncode;
 
 lirc_t signals[MAX_SIGNALS];
 
-#define AEPS 100
-#define EPS 30
+unsigned int eps = 30;
+lirc_t aeps = 100;
 
 /* some threshold values */
 
@@ -119,6 +121,7 @@ lirc_t signals[MAX_SIGNALS];
 #define TH_TRAIL       90
 #define TH_LEAD        90
 #define TH_IS_BIT      15
+#define TH_RC6_SIGNAL 550
 
 #define MIN_GAP  20000
 #define MAX_GAP 100000
@@ -134,31 +137,154 @@ FILE *lf=NULL;
 char *hostname="";
 int daemonized=0;
 
+struct ir_remote *emulation_data;
+struct ir_ncode *next_code = NULL;
+struct ir_ncode *current_code = NULL;
+int current_index = 0;
+int current_rep = 0;
+
+lirc_t emulation_readdata(lirc_t timeout)
+{
+	static lirc_t sum = 0;
+	lirc_t data = 0;
+	
+	if(current_code == NULL)
+	{
+		data = 1000000;
+		if(next_code)
+		{
+			current_code = next_code;
+		}
+		else
+		{
+			current_code = emulation_data->codes;
+		}
+		sum = 0;
+	}
+	else
+	{
+		if(current_code->name == NULL)
+		{
+			fprintf(stderr, "%s: %s no data found\n",
+				progname, emulation_data->name);
+			data = 0;
+		}
+		if(current_index >= current_code->length)
+		{
+			if(next_code)
+			{
+				current_code = next_code;
+			}
+			else
+			{
+				current_rep++;
+				if(current_rep > 2)
+				{
+					current_code++;
+					current_rep = 0;
+					data = 1000000;
+				}
+			}
+			current_index = 0;
+			if(current_code->name == NULL)
+			{
+				current_code = NULL;
+			}
+			if(data == 0)
+			{
+				if(is_const(emulation_data))
+				{
+					data = emulation_data->gap - sum;
+				}
+				else
+				{
+					data = emulation_data->gap;
+				}
+			}
+			
+			sum = 0;
+		}
+		else
+		{
+			data = current_code->signals[current_index];
+			if((current_index % 2) == 0)
+			{
+				data |= PULSE_BIT;
+			}
+			current_index++;
+			sum += data&PULSE_MASK;
+		}
+	}
+	/*
+	printf("delivering: %c%u\n", data&PULSE_BIT ? 'p':'s',
+	       data&PULSE_MASK);
+	*/
+	return data;
+}
+
+struct hardware hw_emulation=
+{
+	"/dev/null",        /* default device */
+	-1,                 /* fd */
+	LIRC_CAN_REC_MODE2, /* features */
+	0,                  /* send_mode */
+	LIRC_MODE_MODE2,    /* rec_mode */
+	0,                  /* code_length */
+	NULL,               /* init_func */
+	NULL,               /* config_func */
+	NULL,               /* deinit_func */
+	NULL,               /* send_func */
+	NULL,               /* rec_func */
+	NULL,               /* decode_func */
+	NULL,               /* ioctl_func */
+	emulation_readdata,
+	"emulation"
+};
+
+static int iprintf(int interactive, char *format_str, ...)
+{
+	va_list ap;
+	int ret = 0;
+	
+#ifndef DEBUG
+	if(interactive)
+#endif
+	{
+		va_start(ap,format_str);
+		ret = vfprintf(stdout, format_str, ap);
+		va_end(ap);
+	}
+	return ret;
+}
+
 void logprintf(int prio,char *format_str, ...)
 {
-	time_t current;
-	char *currents;
 	va_list ap;  
 	
-	current=time(&current);
-	currents=ctime(&current);
-	
-	if(lf) fprintf(lf,"%15.15s %s %s: ",currents+4,hostname,progname);
-	if(!daemonized) fprintf(stderr,"%s: ",progname);
-	va_start(ap,format_str);
 	if(lf)
 	{
+		time_t current;
+		char *currents;
+		
+		current=time(&current);
+		currents=ctime(&current);
+		
+		fprintf(lf,"%15.15s %s %s: ",currents+4,hostname,progname);
+		va_start(ap,format_str);
 		if(prio==LOG_WARNING) fprintf(lf,"WARNING: ");
 		vfprintf(lf,format_str,ap);
 		fputc('\n',lf);fflush(lf);
+		va_end(ap);
 	}
 	if(!daemonized)
 	{
+		fprintf(stderr,"%s: ",progname);
+		va_start(ap,format_str);
 		if(prio==LOG_WARNING) fprintf(stderr,"WARNING: ");
 		vfprintf(stderr,format_str,ap);
 		fputc('\n',stderr);fflush(stderr);
+		va_end(ap);
 	}
-	va_end(ap);
 }
 
 void logperror(int prio,const char *s)
@@ -186,11 +312,14 @@ int main(int argc,char **argv)
 	int retval=EXIT_SUCCESS;
 	ir_code pre,code,post;
 	int repeat_flag;
-	lirc_t remaining_gap;
+	lirc_t min_remaining_gap, max_remaining_gap;
 	int force;
 	int retries;
+	int no_data = 0;
 	struct ir_remote *remotes=NULL;
 	char *device=NULL;
+	int using_template = 0;
+	int analyse = 0;
 #ifdef DEBUG
 	int get_pre=0,get_post=0,test=0,invert=0,trail=0;
 #endif
@@ -205,6 +334,7 @@ int main(int argc,char **argv)
 		{
 			{"help",no_argument,NULL,'h'},
 			{"version",no_argument,NULL,'v'},
+			{"analyse",no_argument,NULL,'a'},			
 			{"device",required_argument,NULL,'d'},
 			{"driver",required_argument,NULL,'H'},
 			{"force",no_argument,NULL,'f'},
@@ -218,9 +348,9 @@ int main(int argc,char **argv)
 			{0, 0, 0, 0}
 		};
 #ifdef DEBUG
-		c = getopt_long(argc,argv,"hvd:H:fpPtiT",long_options,NULL);
+		c = getopt_long(argc,argv,"hvad:H:fpPtiT",long_options,NULL);
 #else
-		c = getopt_long(argc,argv,"hvd:H:f",long_options,NULL);
+		c = getopt_long(argc,argv,"hvad:H:f",long_options,NULL);
 #endif
 		if(c==-1)
 			break;
@@ -230,6 +360,7 @@ int main(int argc,char **argv)
 			printf(usage,progname);
 			printf("\t -h --help\t\tdisplay this message\n");
 			printf("\t -v --version\t\tdisplay version\n");
+			printf("\t -a --analyse\t\tanalyse raw_codes config files\n");
 			printf("\t -f --force\t\tforce raw mode\n");
 			printf("\t -H --driver=driver\tuse given driver\n");
 			printf("\t -d --device=device\tread from given device\n");
@@ -237,6 +368,9 @@ int main(int argc,char **argv)
 		case 'v':
 			printf("irrecord %s\n",IRRECORD_VERSION);
 			exit(EXIT_SUCCESS);
+		case 'a':
+			analyse = 1;
+			break;
 		case 'H':
 			if(hw_choose_driver(optarg) != 0){
 				fprintf(stderr, "Driver `%s' not supported.\n",
@@ -293,13 +427,23 @@ int main(int argc,char **argv)
 			progname);
 		exit(EXIT_FAILURE);
 	}
+	aeps = (hw.resolution>aeps ? hw.resolution:aeps);
 	filename=argv[optind];
 	fin=fopen(filename,"r");
 	if(fin!=NULL)
 	{
 		char *filename_new;
 		
-		remotes=read_config(fin);
+		if(force)
+		{
+			fprintf(stderr,
+				"%s: file \"%s\" already exists\n"
+				"%s: you cannot use the --force option "
+				"together with a template file\n",
+				progname, filename, progname);
+			exit(EXIT_FAILURE);
+		}
+		remotes=read_config(fin, filename);
 		fclose(fin);
 		if(remotes==(void *) -1 || remotes==NULL)
 		{
@@ -308,6 +452,13 @@ int main(int argc,char **argv)
 				"data\n",progname,filename);
 			exit(EXIT_FAILURE);
 		}
+		if(analyse)
+		{
+			hw = hw_emulation;
+			for_each_remote(remotes, analyse_remote);
+			return EXIT_SUCCESS;
+		}
+		using_template = 1;
 #ifdef DEBUG
 		if(test)
 		{
@@ -329,12 +480,11 @@ int main(int argc,char **argv)
 		remote.last_code=NULL;
 		remote.next=NULL;
 		remote.toggle_bit=0;
+		remote.toggle_bit_mask=0;
 		if(remote.pre_p==0 && remote.pre_s==0 &&
 		   remote.post_p==0 && remote.post_s==0)
 		{
-			remote.bits=remote.pre_data_bits+
-				remote.bits+
-				remote.post_data_bits;
+			remote.bits=bit_count(&remote);
 			remote.pre_data_bits=0;
 			remote.post_data_bits=0;
 		}
@@ -352,8 +502,18 @@ int main(int argc,char **argv)
 				progname);
 			exit(EXIT_FAILURE);
 		}
-		strcpy(filename_new,filename);
-		strcat(filename,".conf");
+		strcpy(filename_new, filename);
+		strcat(filename_new, ".conf");
+		filename = filename_new;
+	}
+	else
+	{
+		if(analyse)
+		{
+			fprintf(stderr, "%s: no input file given, "
+				"ignoring analyse flag\n", progname);
+			analyse = 0;
+		}
 	}
 	fout=fopen(filename,"w");
 	if(fout==NULL)
@@ -448,7 +608,7 @@ int main(int argc,char **argv)
 	switch(hw.rec_mode)
 	{
 	case LIRC_MODE_MODE2:
-		if(remotes==NULL && !get_lengths(&remote,force))
+		if(!using_template && !get_lengths(&remote, force, 0))
 		{
 			if(remote.gap==0)
 			{
@@ -461,8 +621,8 @@ int main(int argc,char **argv)
 			}
 			printf("Creating config file in raw mode.\n");
 			set_protocol(&remote, RAW_CODES);
-			remote.eps=EPS;
-			remote.aeps=AEPS;
+			remote.eps=eps;
+			remote.aeps=aeps;
 			break;
 		}
 		
@@ -482,7 +642,7 @@ int main(int argc,char **argv)
 	case LIRC_MODE_LIRCCODE:
 		if(hw.rec_mode==LIRC_MODE_CODE) remote.bits=CHAR_BIT;
 		else remote.bits=hw.code_length;
-		if(!get_gap_length(&remote))
+		if(!using_template && !get_gap_length(&remote))
 		{
 			fprintf(stderr,"%s: gap not found,"
 				" can't continue\n",progname);
@@ -494,14 +654,14 @@ int main(int argc,char **argv)
 		break;
 	}
 	
-	if(is_rc6(&remote))
+	if(!using_template && is_rc6(&remote))
 	{
 		sleep(1);
 		while(availabledata())
 		{
 			hw.rec_func(NULL);
 		}
-		if(!get_toggle_bit(&remote))
+		if(!get_toggle_bit_mask(&remote))
 		{
 			printf("But I know for sure that RC6 has a toggle bit!\n");
 			fclose(fout);
@@ -521,6 +681,15 @@ int main(int argc,char **argv)
 		char buffer[BUTTON];
 		char *string;
 
+		if(no_data)
+		{
+			fprintf(stderr,"%s: no data for 10 secs,"
+				" aborting\n",progname);
+			printf("The last button did not seem to generate any signal.\n");
+			printf("Press RETURN to continue.\n\n");
+			getchar();
+			no_data = 0;
+		}
 		printf("\nPlease enter the name for the next button (press <ENTER> to finish recording)\n");
 		string=fgets(buffer,BUTTON,stdin);
 		
@@ -580,9 +749,7 @@ int main(int argc,char **argv)
 				{
 					if(count==0)
 					{
-						fprintf(stderr,"%s: no data for 10 secs,"
-							" aborting\n",progname);
-						retval=EXIT_FAILURE;
+						no_data = 1;
 						break;
 					}
 					data=remote.gap;
@@ -653,9 +820,7 @@ int main(int argc,char **argv)
 
 			if(!waitfordata(10000000))
 			{
-				fprintf(stderr,"%s: no data for 10 secs,"
-					" aborting\n",progname);
-				retval=EXIT_FAILURE;
+				no_data = 1;
 				break;
 			}
 			last_remote=NULL;
@@ -665,7 +830,9 @@ int main(int argc,char **argv)
 			{
 				hw.rec_func(NULL);
 				if(hw.decode_func(&remote,&pre,&code,&post,
-						  &repeat_flag,&remaining_gap))
+						  &repeat_flag,
+						  &min_remaining_gap,
+						  &max_remaining_gap))
 				{
 					flag=1;
 					break;
@@ -734,7 +901,7 @@ int main(int argc,char **argv)
 		if(hw.deinit_func) hw.deinit_func();
 		exit(EXIT_FAILURE);
 	}
-	remotes=read_config(fin);
+	remotes=read_config(fin,filename);
 	fclose(fin);
 	if(remotes==NULL)
 	{
@@ -753,14 +920,13 @@ int main(int argc,char **argv)
 		exit(EXIT_FAILURE);
 	}
 	
-	if(remotes->toggle_bit==0)
+	if(!has_toggle_bit_mask(remotes))
 	{
-		get_toggle_bit(remotes);
+		if(!using_template) get_toggle_bit_mask(remotes);
 	}
 	else
 	{
-		set_toggle_bit(remotes,
-			       1<<(remotes->bits-remotes->toggle_bit));
+		set_toggle_bit_mask(remotes, remotes->toggle_bit_mask);
 	}
 	if(hw.deinit_func) hw.deinit_func();
 	get_pre_data(remotes);
@@ -896,22 +1062,30 @@ int availabledata(void)
 	return(0);
 }
 
-int get_toggle_bit(struct ir_remote *remote)
+int get_toggle_bit_mask(struct ir_remote *remote)
 {
 	ir_code pre,code,post;
 	int repeat_flag;
-	lirc_t remaining_gap;
+	lirc_t min_remaining_gap, max_remaining_gap;
 	int retval=EXIT_SUCCESS;
 	int retries,flag,success;
-	ir_code first;
+	ir_code first, last;
 	int seq,repeats;
 	int found;
 	
-	printf("Checking for toggle bit.\n");
+	printf("Checking for toggle bit mask.\n");
 	printf(
-"Please press an arbitrary button repeatedly as fast as possible (don't hold\n"
-"it down!).\n");
-	retries=30;flag=success=0;first=0;
+"Please press an arbitrary button repeatedly as fast as possible.\n"
+"Make sure you keep pressing the SAME button and that you DON'T HOLD\n"
+"the button down!.\n"
+"If you can't see any dots appear, then wait a bit between button presses.\n"
+"\n"
+"Press RETURN to continue."
+);
+	fflush(stdout);
+	getchar();
+
+	retries=30;flag=success=0;first=0;last=0;
 	seq=repeats=0;found=0;
 	while(availabledata())
 	{
@@ -937,10 +1111,12 @@ int get_toggle_bit(struct ir_remote *remote)
 				remote->rc6_mask=mask;
 				success=hw.decode_func(remote,&pre,&code,&post,
 						       &repeat_flag,
-						       &remaining_gap);
+						       &min_remaining_gap,
+						       &max_remaining_gap);
 				if(success)
 				{
-					remote->remaining_gap=remaining_gap;
+					remote->min_remaining_gap=min_remaining_gap;
+					remote->max_remaining_gap=max_remaining_gap;
 					break;
 				}
 			}
@@ -949,10 +1125,13 @@ int get_toggle_bit(struct ir_remote *remote)
 		else
 		{
 			success=hw.decode_func(remote,&pre,&code,&post,
-					       &repeat_flag,&remaining_gap);
+					       &repeat_flag,
+					       &min_remaining_gap,
+					       &max_remaining_gap);
 			if(success)
 			{
-				remote->remaining_gap=remaining_gap;
+				remote->min_remaining_gap=min_remaining_gap;
+				remote->max_remaining_gap=max_remaining_gap;
 			}
 		}
 		if(success)
@@ -962,15 +1141,12 @@ int get_toggle_bit(struct ir_remote *remote)
 				flag=1;
 				first=code;
 			}
-			else if(!repeat_flag)
+			else if(!repeat_flag || code!=last)
 			{
 				seq++;
-				if(!found && (is_rc6(remote) || first^code))
+				if(!found && first^code)
 				{
-					if(!is_rc6(remote))
-					{
-						set_toggle_bit(remote,first^code);
-					}
+					set_toggle_bit_mask(remote,first^code);
 					found=1;
 				}
 				printf(".");fflush(stdout);
@@ -980,6 +1156,7 @@ int get_toggle_bit(struct ir_remote *remote)
 			{
 				repeats++;
 			}
+			last = code;
 		}
 		else
 		{
@@ -988,22 +1165,19 @@ int get_toggle_bit(struct ir_remote *remote)
 	}
 	if(!found)
 	{
-		printf("\nNo toggle bit found.\n");
+		printf("\nNo toggle bit mask found.\n");
 	}
 	else
 	{
-		if(remote->toggle_bit>0)
+		if(remote->toggle_bit_mask>0)
 		{
-			printf("\nToggle bit is %d.\n",remote->toggle_bit);
+			printf("\nToggle bit mask is 0x%llx.\n",
+			       (unsigned long long) remote->toggle_bit_mask);
 		}
 		else if(remote->toggle_mask!=0)
 		{
 			printf("\nToggle mask found.\n");
 		}
-		else
-		{
-			printf("\nInvalid toggle bit.\n");
-		}
 	}
 	if(seq>0) remote->min_repeat=repeats/seq;
 #       ifdef DEBUG
@@ -1012,42 +1186,40 @@ int get_toggle_bit(struct ir_remote *remote)
 	return(found);
 }
 
-void set_toggle_bit(struct ir_remote *remote,ir_code xor)
+void set_toggle_bit_mask(struct ir_remote *remote,ir_code xor)
 {
 	ir_code mask;
-	int toggle_bit;
 	struct ir_ncode *codes;
 	int bits;
 
 	if(!remote->codes) return;
 
-	bits=remote->bits+remote->pre_data_bits+remote->post_data_bits;
+	bits=bit_count(remote);
 	mask=((ir_code) 1)<<(bits-1);
-	toggle_bit=1;
 	while(mask)
 	{
 		if(mask==xor) break;
 		mask=mask>>1;
-		toggle_bit++;
 	}
 	if(mask)
 	{
-		remote->toggle_bit=toggle_bit;
+		remote->toggle_bit_mask = xor;
 		
 		codes=remote->codes;
 		while(codes->name!=NULL)
 		{
-			codes->code&=~mask;
+			codes->code&=~xor;
 			codes++;
 		}
 	}
+	/* Sharp, Denon and some others use a toggle_mask */
+	else if(bits==15 && xor==0x3ff)
+	{
+		remote->toggle_mask=xor;
+	}
 	else
 	{
-		/* Sharp, Denon and some others use a toggle_mask */
-		if(bits==15 && xor==0x3ff)
-		{
-			remote->toggle_mask=xor;
-		}
+		remote->toggle_bit_mask = xor;
 	}
 }
 
@@ -1189,7 +1361,6 @@ void get_post_data(struct ir_remote *remote)
 	}
 }
 
-#ifdef DEBUG
 void for_each_remote(struct ir_remote *remotes, remote_func func)
 {
 	struct ir_remote *remote;
@@ -1202,6 +1373,74 @@ void for_each_remote(struct ir_remote *remotes, remote_func func)
 	}
 }
 
+void analyse_remote(struct ir_remote *raw_data)
+{
+	struct ir_ncode *codes;
+	ir_code pre, code, post;
+	int repeat_flag;
+	lirc_t min_remaining_gap, max_remaining_gap;
+	struct ir_ncode new_codes[100];
+	int new_index = 0;
+	int ret;
+
+	if(!is_raw(raw_data))
+	{
+		fprintf(stderr, "%s: remote %s not in raw mode, ignoring\n",
+			progname, raw_data->name);
+		return;
+	}
+	aeps = raw_data->aeps;
+	eps = raw_data->eps;
+	emulation_data = raw_data;
+	next_code = NULL;
+	current_code = NULL;
+	current_index = 0;
+	get_lengths(&remote, 0, 0 /* not interactive */ );
+
+	if(is_rc6(&remote))
+	{
+		/* have to assume something as it's very difficult to
+		   extract the rc6_mask from the data that we have */
+		remote.rc6_mask = (ir_code) 0x100000000ll;
+	}
+
+	remote.name = raw_data->name;
+	
+	memset(new_codes, 0, sizeof(new_codes));
+	codes = raw_data->codes;
+	while(codes->name!=NULL)
+	{
+		//printf("decoding %s\n", codes->name);
+		current_code = NULL;
+		current_index = 0;
+		next_code = codes;
+		
+                init_rec_buffer();
+		
+		ret = receive_decode(&remote,
+				     &pre, &code, &post,
+				     &repeat_flag,
+				     &min_remaining_gap,
+				     &max_remaining_gap);
+		if(!ret)
+		{
+			fprintf(stderr, "%s: decoding of %s failed\n",
+				progname, codes->name);
+		}
+		else
+		{
+			new_codes[new_index].name = codes->name;
+			new_codes[new_index].code = code;
+			new_index++;
+		}
+		codes++;
+	}
+	new_codes[new_index].name = NULL;
+	remote.codes = new_codes;
+	fprint_remotes(stdout, &remote);
+}
+
+#ifdef DEBUG
 void remove_pre_data(struct ir_remote *remote)
 {
 	struct ir_ncode *codes;
@@ -1368,28 +1607,34 @@ inline lirc_t calc_signal(struct lengths *len)
 	return((lirc_t) (len->sum/len->count));
 }
 
-int get_lengths(struct ir_remote *remote,int force)
+int get_lengths(struct ir_remote *remote, int force, int interactive)
 {
 	int retval;
 	lirc_t data,average,sum,remaining_gap,header;
 	enum analyse_mode mode=MODE_GAP;
 	int first_signal;
 
-	printf("Now start pressing buttons on your remote control.\n\n");
-	printf(
+	if(interactive)
+	{
+		printf("Now start pressing buttons on your remote control.\n\n");
+		printf(
 "It is very important that you press many different buttons and hold them\n"
 "down for approximately one second. Each button should generate at least one\n"
 "dot but in no case more than ten dots of output.\n"
 "Don't stop pressing buttons until two lines of dots (2x80) have been\n"
 "generated.\n\n");
-	printf("Press RETURN now to start recording.");fflush(stdout);
-	getchar();
-	flushhw();
+		printf("Press RETURN now to start recording.");fflush(stdout);
+		getchar();
+		flushhw();
+	}
 	retval=1;
 	average=0;sum=0;count=0;count_spaces=0;
 	count_3repeats=0;count_5repeats=0;count_signals=0;
 	first_signal=-1;header=0;
 	first_length=0;
+	first_lengths = 0;
+	second_lengths = 0;
+	memset(lengths, 0, sizeof(lengths));
 	while(1)
 	{
 		data=hw.readdata(10000000);
@@ -1441,7 +1686,7 @@ int get_lengths(struct ir_remote *remote,int force)
 						{
 							remote->gap=calc_signal(scan);
 							remote->flags|=CONST_LENGTH;
-							printf("\nFound const length: %lu\n",(unsigned long) remote->gap);
+							iprintf(interactive, "\nFound const length: %lu\n",(unsigned long) remote->gap);
 							break;
 						}
 						scan=scan->next;
@@ -1456,7 +1701,7 @@ int get_lengths(struct ir_remote *remote,int force)
 							if(scan->count>SAMPLES)
 							{
 								remote->gap=calc_signal(scan);
-								printf("\nFound gap: %lu\n",(unsigned long) remote->gap);
+								iprintf(interactive, "\nFound gap: %lu\n",(unsigned long) remote->gap);
 								break;
 							}
 							scan=scan->next;
@@ -1464,7 +1709,7 @@ int get_lengths(struct ir_remote *remote,int force)
 					}
 					if(scan!=NULL)
 					{
-						printf("Please keep on pressing buttons like described above.\n");
+						iprintf(interactive, "Please keep on pressing buttons like described above.\n");
 						mode=MODE_HAVE_GAP;
 						sum=0;
 						count=0;
@@ -1480,10 +1725,13 @@ int get_lengths(struct ir_remote *remote,int force)
 						continue;
 					}
 					
-					for(i=maxcount-lastmaxcount;i>0;i--)
+					if(interactive)
 					{
-						printf(".");
-						fflush(stdout);
+						for(i=maxcount-lastmaxcount;i>0;i--)
+						{
+							printf(".");
+							fflush(stdout);
+						}
 					}
 					lastmaxcount=maxcount;
 					
@@ -1493,7 +1741,7 @@ int get_lengths(struct ir_remote *remote,int force)
 				/(count_spaces+1);
 				count_spaces++;
 			}
-			if(count>SAMPLES*MAX_SIGNALS)
+			if(count>SAMPLES*MAX_SIGNALS*2)
 			{
 				fprintf(stderr,"\n%s: could not find gap.\n",
 					progname);
@@ -1525,8 +1773,8 @@ int get_lengths(struct ir_remote *remote,int force)
 			}
 			sum+=data&PULSE_MASK;
 
-			if((data&PULSE_MASK)>=remaining_gap*(100-EPS)/100
-			   || (data&PULSE_MASK)>=remaining_gap-AEPS)
+			if((data&PULSE_MASK)>=remaining_gap*(100-eps)/100
+			   || (data&PULSE_MASK)>=remaining_gap-aeps)
 			{
 				if(is_space(data))
 				{
@@ -1563,7 +1811,10 @@ int get_lengths(struct ir_remote *remote,int force)
 					{
 						int i;
 
-						printf(".");fflush(stdout);
+						if(interactive)
+						{
+							printf(".");fflush(stdout);
+						}
 						count_signals++;
 						add_length(&first_1lead,signals[0]);
 						merge_lengths(first_1lead);
@@ -1627,42 +1878,44 @@ int get_lengths(struct ir_remote *remote,int force)
 #endif
 				if(count_signals>=SAMPLES)
 				{
-					printf("\n");
-					get_scheme(remote);
-					if(!get_header_length(remote) ||
-					   !get_trail_length(remote) ||
+					iprintf(interactive, "\n");
+					get_scheme(remote, interactive);
+					if(!get_header_length(remote, interactive) ||
+					   !get_trail_length(remote, interactive) ||
 					   !get_lead_length(remote) ||
-					   !get_repeat_length(remote) ||
-					   !get_data_length(remote))
+					   !get_repeat_length(remote, interactive) ||
+					   !get_data_length(remote, interactive))
 					{
 						retval=0;
 					}
 					break;
 				}
-				if((data&PULSE_MASK)<=(remaining_gap+header)*(100+EPS)/100
-				   || (data&PULSE_MASK)<=(remaining_gap+header)+AEPS)
+				if((data&PULSE_MASK)<=(remaining_gap+header)*(100+eps)/100
+				   || (data&PULSE_MASK)<=(remaining_gap+header)+aeps)
 				{
 					first_signal=0;
 					header=0;
 				}
 				else
+				{
 					first_signal=1;
+				}
 			}
 		}
 	}
-	free_lengths(first_space);
-	free_lengths(first_pulse);
-	free_lengths(first_sum);
-	free_lengths(first_gap);
-	free_lengths(first_repeat_gap);
-	free_lengths(first_signal_length);
-	free_lengths(first_headerp);
-	free_lengths(first_headers);
-	free_lengths(first_1lead);
-	free_lengths(first_3lead);
-	free_lengths(first_trail);
-	free_lengths(first_repeatp);
-	free_lengths(first_repeats);
+	free_lengths(&first_space);
+	free_lengths(&first_pulse);
+	free_lengths(&first_sum);
+	free_lengths(&first_gap);
+	free_lengths(&first_repeat_gap);
+	free_lengths(&first_signal_length);
+	free_lengths(&first_headerp);
+	free_lengths(&first_headers);
+	free_lengths(&first_1lead);
+	free_lengths(&first_3lead);
+	free_lengths(&first_trail);
+	free_lengths(&first_repeatp);
+	free_lengths(&first_repeats);
 	return(retval);
 }
 
@@ -1712,17 +1965,19 @@ int add_length(struct lengths **first,lirc_t length)
 	return(1);
 }
 
-void free_lengths(struct lengths *first)
+void free_lengths(struct lengths **firstp)
 {
-	struct lengths *next;
+	struct lengths *first, *next;
 
-	if(first==NULL) return;
+	first = *firstp;
+	if(first == NULL) return;
 	while(first!=NULL)
 	{
-		next=first->next;
+		next = first->next;
 		free(first);
 		first=next;
 	}
+	*firstp = NULL;
 }
 
 void merge_lengths(struct lengths *first)
@@ -1741,15 +1996,15 @@ void merge_lengths(struct lengths *first)
 			new_sum=l->sum+inner->sum;
 			new_count=l->count+inner->count;
 			
-			if((l->max<=new_sum/new_count+AEPS &&
-			    l->min>=new_sum/new_count-AEPS &&
-			    inner->max<=new_sum/new_count+AEPS &&
-			    inner->min>=new_sum/new_count-AEPS)
+			if((l->max<=new_sum/new_count+aeps &&
+			    l->min+aeps>=new_sum/new_count &&
+			    inner->max<=new_sum/new_count+aeps &&
+			    inner->min+aeps>=new_sum/new_count)
 			   ||
-			   (l->max<=new_sum/new_count*(100+EPS) &&
-			    l->min>=new_sum/new_count*(100-EPS) &&
-			    inner->max<=new_sum/new_count*(100+EPS) &&
-			    inner->min>=new_sum/new_count*(100-EPS)))
+			   (l->max<=new_sum/new_count*(100+eps) &&
+			    l->min>=new_sum/new_count*(100-eps) &&
+			    inner->max<=new_sum/new_count*(100+eps) &&
+			    inner->min>=new_sum/new_count*(100-eps)))
 			{
 				l->sum=new_sum;
 				l->count=new_count;
@@ -1782,7 +2037,7 @@ void merge_lengths(struct lengths *first)
 #       endif
 }
 
-void get_scheme(struct ir_remote *remote)
+void get_scheme(struct ir_remote *remote, int interactive)
 {
 	unsigned int i,length=0,sum=0;
 
@@ -1802,11 +2057,12 @@ void get_scheme(struct ir_remote *remote)
 	       "first_lengths: %lu second_lengths: %lu\n",
 	       sum,length+1,lengths[length],first_lengths,second_lengths);
 #       endif
+	/* FIXME !!! this heuristic is too bad */
 	if(lengths[length]>=TH_SPACE_ENC*sum/100)
 	{
 		length++;
-		printf("Space/pulse encoded remote control found.\n");
-		printf("Signal length is %u.\n",length);
+		iprintf(interactive, "Space/pulse encoded remote control found.\n");
+		iprintf(interactive, "Signal length is %u.\n",length);
 		/* this is not yet the
 		   number of bits */
 		remote->bits=length;
@@ -1843,17 +2099,17 @@ void get_scheme(struct ir_remote *remote)
 				
 				maxs=get_max_length(first_space,NULL);
 				
-				if((calc_signal(maxp)<500 ||
-				    calc_signal(max2p)<500) &&
-				   (calc_signal(maxs)<500 ||
-				    calc_signal(max2s)<500))
+				if((calc_signal(maxp)<TH_RC6_SIGNAL ||
+				    calc_signal(max2p)<TH_RC6_SIGNAL) &&
+				   (calc_signal(maxs)<TH_RC6_SIGNAL ||
+				    calc_signal(max2s)<TH_RC6_SIGNAL))
 				{
-					printf("RC-6 remote control found.\n");
+					iprintf(interactive, "RC-6 remote control found.\n");
 					set_protocol(remote, RC6);
 				}
 				else
 				{
-					printf("RC-5 remote control found.\n");
+					iprintf(interactive, "RC-5 remote control found.\n");
 					set_protocol(remote, RC5);
 				}
 				return;
@@ -1861,7 +2117,7 @@ void get_scheme(struct ir_remote *remote)
 		}
 	}
 	length++;
-	printf("Suspicious data length: %u.\n",length);
+	iprintf(interactive, "Suspicious data length: %u.\n",length);
 	/* this is not yet the number of bits */
 	remote->bits=length;
 	set_protocol(remote, SPACE_ENC);
@@ -1894,7 +2150,7 @@ struct lengths *get_max_length(struct lengths *first,unsigned int *sump)
 	return(max_length);
 }
 
-int get_trail_length(struct ir_remote *remote)
+int get_trail_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	struct lengths *max_length;
@@ -1908,12 +2164,12 @@ int get_trail_length(struct ir_remote *remote)
 #       endif
 	if(max_count>=sum*TH_TRAIL/100)
 	{
-		printf("Found trail pulse: %lu\n",
-		       (unsigned long) calc_signal(max_length));
+		iprintf(interactive, "Found trail pulse: %lu\n",
+			(unsigned long) calc_signal(max_length));
 		remote->ptrail=calc_signal(max_length);
 		return(1);
 	}
-	printf("No trail pulse found.\n");
+	iprintf(interactive, "No trail pulse found.\n");
 	return(1);
 }
 
@@ -1921,7 +2177,7 @@ int get_lead_length(struct ir_remote *remote)
 {
 	unsigned int sum,max_count;
 	struct lengths *first_lead,*max_length,*max2_length;
-	lirc_t a,b;
+	lirc_t a,b,swap;
 	
 	if(!is_biphase(remote)) return(1);
 	if(is_rc6(remote)) return(1);
@@ -1945,8 +2201,11 @@ int get_lead_length(struct ir_remote *remote)
 
 	a=calc_signal(max_length);
 	b=calc_signal(max2_length);
-	if(a>b)	b^=a^=b^=a;
-	if(abs(2*a-b)<b*EPS/100 || abs(2*a-b)<AEPS)
+	if(a>b)
+	{
+		swap=a; a=b; b=swap;
+	}
+	if(abs(2*a-b)<b*eps/100 || abs(2*a-b)<aeps)
 	{
 		printf("Found hidden lead pulse: %lu\n",
 		       (unsigned long) a);
@@ -1957,7 +2216,7 @@ int get_lead_length(struct ir_remote *remote)
 	return(1);
 }
 
-int get_header_length(struct ir_remote *remote)
+int get_header_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	lirc_t headerp,headers;
@@ -1970,7 +2229,7 @@ int get_header_length(struct ir_remote *remote)
 	}
 	else
 	{
-		printf("No header data.\n");
+		iprintf(interactive, "No header data.\n");
 		return(1);
 	}
 #       ifdef DEBUG
@@ -1990,24 +2249,24 @@ int get_header_length(struct ir_remote *remote)
 			headerp=calc_signal(max_plength);
 			headers=calc_signal(max_slength);
 
-			printf("Found possible header: %lu %lu\n",
-			       (unsigned long) headerp,
-			       (unsigned long) headers);
+			iprintf(interactive, "Found possible header: %lu %lu\n",
+				(unsigned long) headerp,
+				(unsigned long) headers);
 			remote->phead=headerp;
 			remote->shead=headers;
 			if(first_lengths<second_lengths)
 			{
-				printf("Header is not being repeated.\n");
+				iprintf(interactive, "Header is not being repeated.\n");
 				remote->flags|=NO_HEAD_REP;
 			}
 			return(1);
 		}
 	}
-	printf("No header found.\n");
+	iprintf(interactive, "No header found.\n");
 	return(1);
 }
 
-int get_repeat_length(struct ir_remote *remote)
+int get_repeat_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	lirc_t repeatp,repeats,repeat_gap;
@@ -2021,7 +2280,7 @@ int get_repeat_length(struct ir_remote *remote)
 			printf("Repeat inconsitentcy.\n");
 			return(0);
 		}
-		printf("No repeat code found.\n");
+		iprintf(interactive, "No repeat code found.\n");
 		return(1);
 	}
 
@@ -2056,9 +2315,9 @@ int get_repeat_length(struct ir_remote *remote)
 			repeatp=calc_signal(max_plength);
 			repeats=calc_signal(max_slength);
 			
-			printf("Found repeat code: %lu %lu\n",
-			       (unsigned long) repeatp,
-			       (unsigned long) repeats);
+			iprintf(interactive, "Found repeat code: %lu %lu\n",
+				(unsigned long) repeatp,
+				(unsigned long) repeats);
 			remote->prepeat=repeatp;
 			remote->srepeat=repeats;
 			if(!(remote->flags&CONST_LENGTH))
@@ -2066,15 +2325,15 @@ int get_repeat_length(struct ir_remote *remote)
 				max_slength=get_max_length(first_repeat_gap,
 							   NULL);
 				repeat_gap=calc_signal(max_slength);
-				printf("Found repeat gap: %lu\n",
-				       (unsigned long) repeat_gap);
+				iprintf(interactive, "Found repeat gap: %lu\n",
+					(unsigned long) repeat_gap);
 				remote->repeat_gap=repeat_gap;
 				
 			}
 			return(1);
 		}
 	}
-	printf("No repeat header found.\n");
+	iprintf(interactive, "No repeat header found.\n");
 	return(1);
 
 }
@@ -2108,7 +2367,7 @@ void unlink_length(struct lengths **first,struct lengths *remove)
 	printf("unlink_length(): report this bug!\n");
 }
 
-int get_data_length(struct ir_remote *remote)
+int get_data_length(struct ir_remote *remote, int interactive)
 {
 	unsigned int sum,max_count;
 	lirc_t p1,p2,s1,s2;
@@ -2170,8 +2429,8 @@ int get_data_length(struct ir_remote *remote)
 #                       endif
 
 
-			remote->eps=EPS;
-			remote->aeps=AEPS;
+			remote->eps=eps;
+			remote->aeps=aeps;
 			if(is_biphase(remote))
 			{
 				if(max2_plength==NULL || max2_slength==NULL)
@@ -2179,7 +2438,7 @@ int get_data_length(struct ir_remote *remote)
 					printf("Unknown encoding found.\n");
 					return(0);
 				}
-				printf("Signals are biphase encoded.\n");
+				iprintf(interactive, "Signals are biphase encoded.\n");
 				p1=calc_signal(max_plength);
 				p2=calc_signal(max2_plength);
 				s1=calc_signal(max_slength);
@@ -2208,7 +2467,7 @@ int get_data_length(struct ir_remote *remote)
 				if(max2_plength)
 				{
 					p2=calc_signal(max2_plength);
-					printf("Signals are pulse encoded.\n");
+					iprintf(interactive, "Signals are pulse encoded.\n");
 					remote->pone=max(p1,p2);
 					remote->sone=s1;
 					remote->pzero=min(p1,p2);
@@ -2222,7 +2481,7 @@ int get_data_length(struct ir_remote *remote)
 				else
 				{
 					s2=calc_signal(max2_slength);
-					printf("Signals are space encoded.\n");
+					iprintf(interactive, "Signals are space encoded.\n");
 					remote->pone=p1;
 					remote->sone=max(s1,s2);
 					remote->pzero=p1;
@@ -2241,7 +2500,7 @@ int get_data_length(struct ir_remote *remote)
 				{
 					remote->phead=remote->shead=0;
 					remote->flags&=~NO_HEAD_REP;
-					printf("Removed header.\n");
+					iprintf(interactive, "Removed header.\n");
 				}
 				if(is_biphase(remote) &&
 				   expect(remote,remote->shead,remote->sone))
@@ -2249,7 +2508,7 @@ int get_data_length(struct ir_remote *remote)
 					remote->plead=remote->phead;
 					remote->phead=remote->shead=0;
 					remote->flags&=~NO_HEAD_REP;
-					printf("Removed header.\n");
+					iprintf(interactive, "Removed header.\n");
 				}
 			}
 			if(is_biphase(remote))
@@ -2276,12 +2535,12 @@ int get_data_length(struct ir_remote *remote)
 					      (has_header(remote) ? 2:0)+
 					      1-(remote->ptrail>0 ? 2:0))/2;
 			}
-			printf("Signal length is %d\n",remote->bits);
-			free_lengths(max_plength);
-			free_lengths(max_slength);
+			iprintf(interactive, "Signal length is %d\n",remote->bits);
+			free_lengths(&max_plength);
+			free_lengths(&max_slength);
 			return(1);
 		}
-		free_lengths(max_plength);
+		free_lengths(&max_plength);
 	}
 	printf("Could not find data lengths.\n");
 	return(0);
@@ -2290,14 +2549,14 @@ int get_data_length(struct ir_remote *remote)
 int get_gap_length(struct ir_remote *remote)
 {
 	struct lengths *gaps=NULL;
-	struct timeval start,end,last;
+	struct timeval start,end,last={0,0};
 	int count,flag;
 	struct lengths *scan;
 	int maxcount,lastmaxcount;
 	lirc_t gap;
 	
-	remote->eps=EPS;
-	remote->aeps=AEPS;
+	remote->eps=eps;
+	remote->aeps=aeps;
 
 	count=0;flag=0;lastmaxcount=0;
 	printf("Hold down an arbitrary button.\n");
@@ -2309,7 +2568,7 @@ int get_gap_length(struct ir_remote *remote)
 		}
 		if(!waitfordata(10000000))
 		{
-			free_lengths(gaps);
+			free_lengths(&gaps);
 			return(0);
 		}
 		gettimeofday(&start,NULL);
@@ -2336,7 +2595,7 @@ int get_gap_length(struct ir_remote *remote)
 					remote->gap+=100000;
 					printf("\nFound gap length: %lu\n",
 					       (unsigned long) remote->gap);
-					free_lengths(gaps);
+					free_lengths(&gaps);
 					return(1);
 				}
 				scan=scan->next;
